"""Stream type classes for tap-aftership."""

from pathlib import Path

import requests
from singer_sdk.pagination import BaseAPIPaginator, BasePageNumberPaginator

from tap_aftership.client import AfterShipStream

SCHEMAS_DIR = Path(__file__).parent / Path("./schemas")


class TrackingsPaginator(BasePageNumberPaginator):
    """Paginator for the trackings AfterShip stream."""

    def has_more(self, response: requests.Response) -> bool:
        """Check if the trackings endpoint has more pages left."""
        # Only return a next page if this is a successful response with more trackings
        # to process
        response_json = response.json()
        if (
            "data" in response_json
            and "trackings" in response_json["data"]
            and response_json["data"]["trackings"]
            and response.status_code == 200
        ):
            return True
        else:
            return False


class TrackingsStream(AfterShipStream):
    """Define custom stream."""

    name = "trackings"
    path = "/v4/trackings"
    primary_keys = ["id"]
    replication_key = None

    schema_filepath = SCHEMAS_DIR / "tracking.json"
    records_jsonpath = "$.data.trackings[*]"

    def get_new_paginator(self) -> BaseAPIPaginator:
        """Get a new TrackingsPaginator for the trackings API endpoint."""
        return TrackingsPaginator(start_value=1)



"""REST client handling, including AfterShipStream base class."""

from pathlib import Path
from typing import Any, Dict, Optional

from singer_sdk.authenticators import APIKeyAuthenticator
from singer_sdk.streams import RESTStream

SCHEMAS_DIR = Path(__file__).parent / Path("./schemas")


class AfterShipStream(RESTStream):
    """AfterShip stream class."""

    url_base = "https://api.aftership.com"
    records_jsonpath = "$[*]"  # Or override `parse_response`.
    next_page_token_jsonpath = "$.data.page"

    @property
    def authenticator(self) -> APIKeyAuthenticator:
        """Return a new authenticator object."""
        return APIKeyAuthenticator.create_for_stream(
            self,
            key="aftership-api-key",
            value=str(self.config.get("api_key")),
            location="header",
        )

    def get_url_params(
        self, context: Optional[dict], next_page_token: Optional[Any]
    ) -> Dict[str, Any]:
        """Return a dictionary of values to be used in URL parameterization."""
        params: dict = {}
        if next_page_token:
            params["page"] = next_page_token
        if "end_date" in self.config:
            params["updated_at_max"] = self.config["end_date"]
        if "start_date" in self.config:
            params["updated_at_min"] = self.config["start_date"]

        return params



"""AfterShip tap class."""

from typing import List

from singer_sdk import Stream, Tap
from singer_sdk import typing as th  # JSON schema typing helpers

from tap_aftership.streams import TrackingsStream

STREAM_TYPES = [
    TrackingsStream,
]


class TapAfterShip(Tap):
    """AfterShip tap class."""

    name = "tap-aftership"

    config_jsonschema = th.PropertiesList(
        th.Property(
            "api_key",
            th.StringType,
            required=True,
            secret=True,  # Flag config as protected.
            description="The token to authenticate against the API service",
        ),
        th.Property(
            "start_date",
            th.DateTimeType,
            description="The earliest record date to sync",
        ),
        th.Property(
            "end_date",
            th.DateTimeType,
            description="The latest record date to sync",
        ),
    ).to_dict()

    def discover_streams(self) -> List[Stream]:
        """Return a list of discovered streams."""
        return [stream_class(tap=self) for stream_class in STREAM_TYPES]


if __name__ == "__main__":
    TapAfterShip.cli()



"""Tests standard tap features using the built-in SDK tests library."""

import requests_mock
from singer_sdk.testing import get_standard_tap_tests

from tap_aftership.streams import TrackingsStream
from tap_aftership.tap import TapAfterShip

from . import SAMPLE_CONFIG


# Run standard built-in tap tests from the SDK:
def test_standard_tap_tests():
    """Run standard tap tests from the SDK."""
    with requests_mock.Mocker() as m:
        path = (
            f"{TrackingsStream.path}?"
            f"updated_at_max={SAMPLE_CONFIG['end_date']}"
            f"&updated_at_min={SAMPLE_CONFIG['start_date']}"
        )
        m.get(path, status_code=200, text='{"data": {"page": 1, "trackings": []}}')
        m.get(
            f"{path}&page=2",
            status_code=400,
            text='{"data": {"page": 2, "trackings": []}}',
        )
        tests = get_standard_tap_tests(TapAfterShip, config=SAMPLE_CONFIG)
        for test in tests:
            test()



import unittest.mock as mock

from tap_aftership.streams import TrackingsPaginator


def test_paginator_has_more():
    mock_response = mock.MagicMock()
    mock_response.status_code = 200
    mock_response.json.return_value = {"data": {"trackings": ["not_empty"]}, "page": 1}

    p = TrackingsPaginator(1)
    assert p.has_more(mock_response)


def test_paginator_no_more_items():
    mock_response = mock.MagicMock()
    mock_response.status_code = 200
    mock_response.json.return_value = {"data": {"trackings": []}, "page": 1}

    p = TrackingsPaginator(1)
    assert not p.has_more(mock_response)


def test_high_page_number():
    mock_response = mock.MagicMock()
    mock_response.status_code = 400
    mock_response.json.return_value = {"data": {"trackings": ["not_empty"]}, "page": 1}

    p = TrackingsPaginator(1)
    assert not p.has_more(mock_response)



"""Test suite for tap-aftership."""
import datetime

now = datetime.datetime.now(datetime.timezone.utc)
SAMPLE_CONFIG = {
    "start_date": (now - datetime.timedelta(days=1)).strftime("%Y-%m-%d"),
    "end_date": now.strftime("%Y-%m-%d"),
    "api_key": "test-api-key",
}



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['tap_aftership', 'tap_aftership.tests']

package_data = \
{'': ['*'], 'tap_aftership': ['schemas/*']}

install_requires = \
['requests>=2.25.1,<3.0.0', 'singer-sdk>=0.13.0,<0.14.0']

entry_points = \
{'console_scripts': ['tap-aftership = tap_aftership.tap:TapAfterShip.cli']}

setup_kwargs = {
    'name': 'tap-aftership',
    'version': '0.0.1',
    'description': '`tap-aftership` is a Singer tap for AfterShip, built with the Meltano Singer SDK.',
    'long_description': 'None',
    'author': 'harrystech',
    'author_email': 'None',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7.1,<3.11',
}


setup(**setup_kwargs)
